#include "AgendaIntegration.hpp"
#include "Generated/agendav1integrationSOAPProxy.h"

using namespace agendav1;

namespace agendav1
{
   #include "Generated/integrationSOAP.nsmap"
}

bool AgendaIntegration::DeleteUser::PerformCall(Int32& status)
{
   /* Prepare service */
   integrationSOAP service;

   /* Assign namespaces */
   service.soap->namespaces = agendav1::namespaces;

   /* Configure ssl stuff */
   soap_ssl_init();
   soap_ssl_client_context(service.soap,
                           SOAP_SSL_DEFAULT,
                           NULL,
                           NULL,
                           cert_path.GetCString(),
                           NULL,
                           NULL);

   /* Prepare result */
   _ns1__SuppressUserResponse result;

   /* Prepare parameters */
   _ns1__SuppressUser parameter;

   ns1__LoginInfo login_info;
   parameter.login_USCOREinfo = &login_info;

   login_info.client_USCOREid = param.account_info.client_id.GetId();
   login_info.password = param.account_info.client_password.GetCString();

   parameter.user_USCORElogin = param.delete_user_login.GetCString();

   /* Perform the request */
   if (service.__ns1__SuppressUser(&parameter, &result) == SOAP_OK)
   {
      /* Get the certificate */
      status = Int32(result.status);
      return true;
   }
   else
   {
      return false;
   }
}

bool AgendaIntegration::ResetUserPassword::PerformCall(Int32& status)
{
   /* Prepare service */
   integrationSOAP service;

   /* Assign namespaces */
   service.soap->namespaces = agendav1::namespaces;

   /* Configure ssl stuff */
   soap_ssl_init();
   soap_ssl_client_context(service.soap,
                           SOAP_SSL_DEFAULT,
                           NULL,
                           NULL,
                           cert_path.GetCString(),
                           NULL,
                           NULL);

   /* Prepare result */
   _ns1__ResetUserPasswordResponse result;

   /* Prepare parameters */
   _ns1__ResetUserPassword parameter;

   ns1__LoginInfo login_info;
   parameter.login_USCOREinfo = &login_info;

   login_info.client_USCOREid = param.account_info.client_id.GetId();
   login_info.password = param.account_info.client_password.GetCString();

   parameter.user_USCORElogin = param.reset_user_login.GetCString();

   /* Perform the request */
   if (service.__ns1__ResetUserPassword(&parameter, &result) == SOAP_OK)
   {
      status = Int32(result.status);
      return true;
   }
   else
   {
      return false;
   }
}

bool AgendaIntegration::CreateUser::PerformCall(Int32& status)
{
   /* Prepare service */
   integrationSOAP service;

   /* Assign namespaces */
   service.soap->namespaces = agendav1::namespaces;

   /* Configure ssl stuff */
   soap_ssl_init();
   soap_ssl_client_context(service.soap,
                           SOAP_SSL_DEFAULT,
                           NULL,
                           NULL,
                           cert_path.GetCString(),
                           NULL,
                           NULL);

   /* Prepare result */
   _ns1__InsertUserResponse result;

   /* Prepare parameters */
   _ns1__InsertUser parameter;

   ns1__LoginInfo login_info;
   parameter.login_USCOREinfo = &login_info;

   login_info.client_USCOREid = param.account_info.client_id.GetId();
   login_info.password = param.account_info.client_password.GetCString();

   parameter.user_USCORElogin = param.create_user_login.GetCString();
   parameter.user_USCOREdisplayed_USCOREname = param.create_user_displayed_name.GetCString();
   parameter.user_USCOREemail = param.create_user_email.GetCString();

   /* Perform the request */
   if (service.__ns1__InsertUser(&parameter, &result) == SOAP_OK)
   {
      status = Int32(result.status);
      return true;
   }
   else
   {
      return false;
   }
}

bool AgendaIntegration::ConfigureAccount::PerformCall(ConfigureAccountResult_t& result)
{
   /* Prepare service */
   integrationSOAP service;

   /* Assign namespaces */
   service.soap->namespaces = agendav1::namespaces;

   /* Configure ssl stuff */
   soap_ssl_init();
   soap_ssl_client_context(service.soap,
                           SOAP_SSL_DEFAULT,
                           NULL,
                           NULL,
                           cert_path.GetCString(),
                           NULL,
                           NULL);

   /* Prepare result */
   _ns1__ConfigureAccountResponse response;

   /* Prepare parameters */
   _ns1__ConfigureAccount parameter;
   parameter.domain = param.domain.GetCString();
   parameter.user = param.user_name.GetCString();
   parameter.password = param.password.GetCString();

   /* Perform the request */
   if (service.__ns1__ConfigureAccount(&parameter, &response) == SOAP_OK)
   {
      /* Retrieve status */
      result.status = (response.status);

      /* Check if we could get result */
      if (result.status == 0)
      {
         result.client_id = *response.client_USCOREid;

         for(std::vector<ns1__AccessibleAgendaInfo*>::iterator it = response.accessible_USCOREagendas.begin(); it != response.accessible_USCOREagendas.end(); ++it)
         {
            ns1__AccessibleAgendaInfo* ptr = *it;

            AccessibleAgendaInfo_t& entry = result.accessible_agendas.AddElement(AccessibleAgendaInfo_t());
            entry.agenda_id = ptr->agenda_USCOREid;
            entry.agenda_name = MyString(wxString(ptr->agenda_USCOREname));
            entry.owner_id = ptr->owner_USCOREid;
         }

         for(std::vector<int>::iterator it = response.writable_USCOREagendas_USCOREids.begin(); it != response.writable_USCOREagendas_USCOREids.end(); ++it)
         {
            int writable_agenda_id = *it;
            result.writable_agendas_ids.AddElement(writable_agenda_id);
         }

         for(std::vector<ns1__OwnerAgendaEntryType*>::iterator it = response.owners_USCOREentry_USCOREtypes.begin(); it != response.owners_USCOREentry_USCOREtypes.end(); ++it)
         {
            ns1__OwnerAgendaEntryType* ptr = *it;

            OwnerEntryTypesInfo_t& entry = result.owners_entry_types.AddElement(OwnerEntryTypesInfo_t());

            entry.owner_id = ptr->owner_USCOREid;
            entry.period_time = ptr->period_USCOREtime;

            for(std::vector<ns1__RdvType*>::iterator it = ptr->rdv_USCOREtypes.begin(); it != ptr->rdv_USCOREtypes.end(); ++it)
            {
               ns1__RdvType* rdv_type = *it;

               RdvTypeInfo_t& e = entry.rdv_types.AddElement(RdvTypeInfo_t());
               e.name = MyString(wxString(rdv_type->name));
               e.nb_periods = rdv_type->nb_USCOREperiods;
            }
         }
      }

      return true;
   }
   else
   {
      return false;
   }
}
