#ifndef AGENDA_CLIENT_HPP
#define AGENDA_CLIENT_HPP

#include "Cpp/Utilities/SoapInvocator.hpp"

#include "Cpp/Types/MyString.hpp"
#include "Cpp/Types/Int32.hpp"
#include "Cpp/Types/MyDate.hpp"
#include "Cpp/Types/DbId.hpp"

#include "Cpp/Containers/MyArray.hpp"

class AgendaIntegration
{
   public:

      typedef struct
      {
         DbId client_id;
         MyString client_password;
      } AccountInfo_t;

      typedef struct
      {
         AccountInfo_t account_info;
         MyString delete_user_login;
      } DeleteUserInfo_t;

      typedef struct
      {
         AccountInfo_t account_info;
         MyString reset_user_login;
      } ResetUserInfo_t;

      typedef struct
      {
         AccountInfo_t account_info;
         MyString create_user_login;
         MyString create_user_displayed_name;
         MyString create_user_email;
      } CreateUserInfo_t;

      typedef struct
      {
         MyString domain;
         MyString user_name;
         MyString password;
      } ConfigureAccountInfo_t;

      typedef struct
      {
         int32 agenda_id;
         int32 owner_id;
         MyString agenda_name;
      } AccessibleAgendaInfo_t;

      typedef struct
      {
         MyString name;
         int32 nb_periods;
      } RdvTypeInfo_t;

      typedef struct
      {
         int32 owner_id;
         int32 period_time;
         MyArray<RdvTypeInfo_t> rdv_types;
      } OwnerEntryTypesInfo_t;

      typedef struct
      {
         int32 status;

         int32 client_id;
         MyArray<AccessibleAgendaInfo_t> accessible_agendas;
         MyArray<int32> writable_agendas_ids;
         MyArray<OwnerEntryTypesInfo_t> owners_entry_types;
      } ConfigureAccountResult_t;

      class DeleteUser: public SoapInvocator<DeleteUserInfo_t,Int32>
      {
         public:

            DeleteUser(const Callback<Int32>& on_success_callback = Callback<Int32>(),
                       const Callback<bool>& on_failure_callback = Callback<bool>())
                  :SoapInvocator(on_success_callback, on_failure_callback)
            {}

         private:

            bool PerformCall(Int32& status);
      };
      class ResetUserPassword: public SoapInvocator<ResetUserInfo_t,Int32>
      {
         public:

            ResetUserPassword(const Callback<Int32>& on_success_callback = Callback<Int32>(),
                              const Callback<bool>& on_failure_callback = Callback<bool>())
                  :SoapInvocator(on_success_callback, on_failure_callback)
            {}

         private:

            bool PerformCall(Int32& status);
      };
      class CreateUser: public SoapInvocator<CreateUserInfo_t,Int32>
      {
         public:

            CreateUser(const Callback<Int32>& on_success_callback = Callback<Int32>(),
                       const Callback<bool>& on_failure_callback = Callback<bool>())
                  :SoapInvocator(on_success_callback, on_failure_callback)
            {}

         private:

            bool PerformCall(Int32& status);
      };
      class ConfigureAccount: public SoapInvocator<ConfigureAccountInfo_t,ConfigureAccountResult_t>
      {
         public:

            ConfigureAccount(const Callback<ConfigureAccountResult_t>& on_success_callback = Callback<ConfigureAccountResult_t>(),
                             const Callback<bool>& on_failure_callback = Callback<bool>())
                  :SoapInvocator(on_success_callback, on_failure_callback)
            {}

         private:

            bool PerformCall(ConfigureAccountResult_t& status);
      };
};

#endif /* AGENDA_CLIENT_HPP */
